@@ -9,6 +9,8 @@ except ImportError:
     import queue as Queue
 import time
 
+import logging
+logger = logging.getLogger('Telebot')
 import re
 from telebot import apihelper, types
 
@@ -83,12 +85,12 @@ class TeleBot:
         :param token: bot API token
         :param create_threads: Create thread for message handler
         :param num_threads: Number of worker in thread pool.
-        :return:
+        :return: Telebot object.
         """
         self.token = token
         self.update_listener = []
         self.polling_thread = None
-        self.__stop_polling = False
+        self.__stop_polling = threading.Event()
         self.last_update_id = 0
         self.num_threads = num_threads
         self.__create_threads = create_threads
@@ -110,7 +112,7 @@ class TeleBot:
                 self.last_update_id = update['update_id']
             msg = types.Message.de_json(update['message'])
             new_messages.append(msg)
-
+        logger.debug('GET %d new messages' % len(new_messages))
         if len(new_messages) > 0:
             self.process_new_messages(new_messages)
 
@@ -136,30 +138,29 @@ class TeleBot:
         :param none_stop: Do not stop polling when Exception occur.
         :return:
         """
-        self.__stop_polling = True
+        self.__stop_polling.set()
         if self.polling_thread:
             self.polling_thread.join()  # wait thread stop.
-        self.__stop_polling = False
+        self.__stop_polling.clear()
         self.polling_thread = threading.Thread(target=self.__polling, args=([none_stop, interval]))
         self.polling_thread.daemon = True
         self.polling_thread.start()
 
     def __polling(self, none_stop, interval):
-        print('TeleBot: Started polling.')
-        while not self.__stop_polling:
+        logger.info('TeleBot: Started polling.')
+        while not self.__stop_polling.wait(interval):
             try:
                 self.get_update()
             except Exception as e:
                 if not none_stop:
-                    self.__stop_polling = True
-                    print("TeleBot: Exception occurred. Stopping.")
-                print(e)
-            time.sleep(interval)
+                    self.__stop_polling.set()
+                    logger.info("TeleBot: Exception occurred. Stopping.")
+                logger.error(e.message)
 
-        print('TeleBot: Stopped polling.')
+        logger.info('TeleBot: Stopped polling.')
 
     def stop_polling(self):
-        self.__stop_polling = True
+        self.__stop_polling.set()
 
     def set_update_listener(self, listener):
         self.update_listener.append(listener)
@@ -472,3 +473,4 @@ class AsyncTeleBot(TeleBot):
     @async()
     def send_chat_action(self, *args, **kwargs):
         return TeleBot.send_chat_action(self, *args, **kwargs)
+
